#!/usr/bin/tclsh

if {[llength $argv] == 0} {
    puts stderr "Usage: convert2mathjax.tcl filename.tex"
    exit 1
}

set infile [lindex $argv 0]
set outfile [file rootname $infile].html

proc WritePreamble {ch title} {
#    puts $ch "<html>
#<head>
#<title>$title</title>
#<script type=\"text/javascript\" async src=\"https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.7/MathJax.js?config=TeX-AMS_SVG\"></script>"
    
#    puts $ch "<script type=\"text/x-mathjax-config\">
#MathJax.Hub.Config({
#    extensions: \[\"tex2jax.js\", \"TeX/AMSmath.js\"\],
#    jax: \[\"input/TeX\",\"output/SVG\"\]
#});
#</script>
#<script type=\"text/javascript\" id=\"MathJax-script\" src=\"/mathjax/tex-chtml.js\"></script>"
    
#    puts $ch "<style type=\"text/css\">
#dt {
#    float: left;
#    clear: left;
#    width: 3em;
#    text-align: right;
#}
#dd {
#    margin: 0 0 0 4em;
#    padding: 0 0 1em 0;
#}
#</style>
#</head>
#<body>

    puts $ch "<!-- Macro definitions -->
\\(
\\def\\ds{\\displaystyle}
\\def\\R{{\\boldsymbol{R}}}
\\def\\m#1{\\begin{pmatrix}#1\\end{pmatrix}}
\\def\\P{\\operatorname{P}}
\\def\\E{\\operatorname{E}}
\\def\\Var{\\operatorname{Var}}
\\def\\cov{\\operatorname{Cov}}
\\def\\Ker{\\operatorname{Ker}}
\\def\\Im{\\operatorname{Im}}
\\def\\tg{\\operatorname{tg}}
\\def\\ctg{\\operatorname{ctg}}
\\def\\arctg{\\operatorname{arctg}}
\\def\\ge{\\geqslant}
\\def\\le{\\leqslant}
\\def\\lim{\\operatorname*{lim}\\limits}
\\)
<h1>$title</h1>
<form action=\"store.yaws\" method=\"POST\">
"
}

proc WritePostamble {ch} {
    puts $ch "<input type=\"hidden\"/ name=\"finished\" value=\"1\"></div>"
    puts $ch "<div class=\"submit\"><input type=\"submit\"/ value=\"Завершить тест\"></div>"
    puts $ch "</form>"
    #puts $ch "</body></html>"
}

proc FindTitle {data} {
    if {![regexp {olympics=(true|false)} $data -> olympics]} {
        set olympics false
    }
    if {![regexp {applied=(true|false)} $data -> applied]} {
        set applied false
    }
    switch -- $olympics {
        true {
            set title "Олимпиада по математике"
        }
        default {
            set title "Экзамен по математике"
        }
    }
    switch -- $applied {
        true {
            append title " для прикладных программ"
        }
        default {
            append title " для программы МАЭ"
        }
    }
    if {[regexp {\\examhead[^\{]*\{([^\}]*)\}} $data -> date]} {
        if {$date eq "\\YEAR"} {
            regexp {\\YEAR\{(\d+)\}} $data -> date
        }
        append title " ($date)"
    }

    return [list $applied $title]
}

proc FixMath {text} {
    set text [regsub -all {\\\$} $text {@DOLLAR@}]
    set text [regsub -all {\$\$(.*?)\$\$} $text {\\[\1\\]}]
    set text [regsub -all {\$(.*?)\$} $text {\\(\1\\)}]
    set text [string map {@DOLLAR@ \$
                          ~ &nbsp;
                          --- "&mdash;"
                          -- "&ndash;"
                          << "&laquo;"
                          >> "&raquo;"
                          < "&lt;"
                          \\, " "
                          \\% %
                          \\dots "..."
                          \\allfalse "все четыре утверждения A, B, C, D ложные"
                          \\existfalse "среди утверждений A, B, C, D есть ложное"
                          \\otherornonexistent "числу, отличному от перечисленных в A, B, C, D, или не существует"
                          \\other "числу, отличному от перечисленных в A, B, C, D"} $text]
    set text [regsub -all {\\(textit|emph)\{(.*?)\}} $text {<i>\2</i>}]
    return $text
}

proc ProcessGRETest {ch num test} {
    if {![regexp {(.*)\\item(\[\+\])?(.*)\\item(\[\+\])?(.*)\\item(\[\+\])?(.*)\\item(\[\+\])?(.*)\\item(\[\+\])?(.*)} \
                 $test -> preamble cA aA cB aB cC aC cD aD cE aE]} {
        exit 1
    }

    puts $ch "<div class=\"test\"><p><b>$num.</b>"

    set fixedPreamble [string trim [FixMath $preamble]]
    if {[regexp {(.*)\\begin\{vars\}.*\\item(.*)\\item(.*)\\item(.*)\\end\{vars\}} \
                $fixedPreamble -> newPreamble i1 i2 i3]} {
        
        puts $ch [string trim $newPreamble]
        puts $ch "</p><ol>"
        foreach i {1 2 3} {
            puts $ch "<li>[string trim [set i$i]]</li>"
        }
        puts $ch "</ol><ul>"
    } else {
        puts $ch $fixedPreamble
        puts $ch "</p><ul>"
    }
    foreach i {A B C D E} {
        #puts -nonewline $ch "<dt>$i</dt>"
        puts $ch "<li><label class=\"my${i}Checkbox\"><input type=\"checkbox\" name=\"gretest$i$num\"/><span></span></label> [FixMath [string trim [set a$i]]]</li>"
    }
    puts $ch "</ul></div>"
    puts $ch ""

    return [list $cA $cB $cC $cD $cE]
}

proc ProcessTest {ch num test} {
    if {![regexp {(.*)\n\n(\\item(\[.\])?(.*)\\item(\[.\])?(.*)\\item(\[.\])?(.*)\\item(\[.\])?(.*))?\\item(\[.\])?(.*)\\item(\[.\])?(.*)\\item(\[.\])?(.*)\\item(\[.\])?(.*)} \
                 $test -> preamble - c1 a1 c2 a2 c3 a3 c4 a4 c5 a5 c6 a6 c7 a7 c8 a8]} {
        exit 1
    }

    array set azbuk {1 а 2 б 3 в 4 г 5 д 6 е 7 ж 8 з}

    puts $ch "<div class=\"test\"><p><b>$num.</b>"
    puts $ch [string trim [FixMath $preamble]]
    puts $ch "</p><ul>"
    set j 0
    foreach i {1 2 3 4 5 6 7 8} {
        if {[set a$i] ne ""} {
            incr j
            puts -nonewline $ch "<li><b>$azbuk($j))</b> "
            puts $ch "&nbsp; [FixMath [string trim [set a$i] " \t\n;."]]<br/>"
            puts $ch "<label class=\"myYCheckbox\"><input type=\"checkbox\" name=\"testY$num${j}\"/><span></span></label>"
            puts $ch "<label class=\"myNCheckbox\"><input type=\"checkbox\" name=\"testN$num${j}\"/><span></span></label></li>"
        }
    }
    puts $ch "</dl></div>"
    puts $ch ""

    return [list $c1 $c2 $c3 $c4 $c5 $c6 $c7 $c8]
}

set fd [open $infile]
set indata [read $fd]
close $fd

set ch [open $outfile w]

lassign [FindTitle $indata] applied title

WritePreamble $ch $title

if {!$applied} {
    puts $ch "<h2>Первая часть</h2>"
    puts $ch "<p><i>Выберите мышкой тот единственный ответ (из A, B, C, D, E), который вы
    считаете правильным. Каждый правильный ответ оценивается в одно очко.
    Неправильный &mdash; в минус четверть очка. Отсутствие ответа &mdash; ноль очков."
    puts $ch "</i></p>"
} else {
    puts $ch "<p><i>Выберите мышкой тот единственный ответ (из A, B, C, D, E), который вы
    считаете правильным. Каждый правильный ответ оценивается в одно очко.
    Неправильный ответ или отсутствие ответа &mdash; ноль очков."
    puts $ch "</i></p>"

}

set current 0
foreach {match test} [regexp -all -inline {\\begin\{gretest\}(.*?)\\end\{gretest\}} $indata] {
    ProcessGRETest $ch [incr current] $test
}

if {!$applied} {
    puts $ch "<h2>Вторая часть</h2>"
    puts $ch "<p><i>Выберите мышкой ответ (из Да, Нет), который вы
    считаете правильным. Каждый правильный ответ оценивается в одно очко.
    Неправильный &mdash; в минус одно очко. Отсутствие ответа &mdash; ноль очков."
    puts $ch "</i></p>"

    set current 0
    foreach {match test} [regexp -all -inline {\\begin\{test\}(.*?)\\end\{test\}} $indata] {
        ProcessTest $ch [incr current] $test
    }
}

WritePostamble $ch
close $ch

